# -*- coding: utf-8 -*-
# Copyright (C) 1998-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls import include
from django.urls import re_path

from postorius.views import domain as domain_views
from postorius.views import list as list_views
from postorius.views import rest as rest_views
from postorius.views import system as system_views
from postorius.views import template as template_views
from postorius.views import user as user_views


list_patterns = [
    re_path(r'^csv_view/$', list_views.csv_view, name='csv_view'),
    re_path(
        r'^members/options/(?P<email>.+)$',
        list_views.list_member_options,
        name='list_member_options',
    ),
    re_path(
        r'^members/(?P<role>\w+)/$',
        list_views.ListMembersViews.as_view(),
        name='list_members',
    ),
    re_path(r'^$', list_views.ListSummaryView.as_view(), name='list_summary'),
    re_path(
        r'^subscribe$',
        list_views.ListSubscribeView.as_view(),
        name='list_subscribe',
    ),
    re_path(
        r'^anonymous_subscribe$',
        list_views.ListAnonymousSubscribeView.as_view(),
        name='list_anonymous_subscribe',
    ),
    re_path(
        r'^change_subscription$',
        list_views.ChangeSubscriptionView.as_view(),
        name='change_subscription',
    ),
    re_path(
        r'^unsubscribe/$',
        list_views.ListUnsubscribeView.as_view(),
        name='list_unsubscribe',
    ),
    re_path(
        r'^subscription_requests$',
        list_views.list_subscription_requests,
        name='list_subscription_requests',
    ),
    re_path(
        r'^unsubscription_requests$',
        list_views.list_unsubscription_requests,
        name='list_unsubscription_requests',
    ),
    re_path(
        r'^pending_confirmation$',
        list_views.list_pending_confirmations,
        name='list_pending_confirmation',
    ),
    re_path(
        r'^handle_subscription_request/(?P<request_id>[^/]+)/$',
        list_views.handle_subscription_request,
        name='handle_subscription_request',
    ),
    re_path(
        r'^mass_subscribe/$',
        list_views.list_mass_subscribe,
        name='mass_subscribe',
    ),
    re_path(
        r'^mass_removal/$',
        list_views.ListMassRemovalView.as_view(),
        name='mass_removal',
    ),
    re_path(r'^delete$', list_views.list_delete, name='list_delete'),
    re_path(
        r'^held_messages$',
        list_views.list_moderation,
        name='list_held_messages',
    ),
    re_path(
        r'^held_messages/moderate$',
        list_views.moderate_held_message,
        name='moderate_held_message',
    ),
    re_path(r'^bans/$', list_views.list_bans, name='list_bans'),
    re_path(
        r'^header-matches/$',
        list_views.list_header_matches,
        name='list_header_matches',
    ),
    re_path(
        r'^remove/(?P<role>[^/]+)/(?P<address>.+)$',
        list_views.remove_role,
        name='remove_role',
    ),
    re_path(
        r'^settings/(?P<visible_section>[^/]+)?$',
        list_views.list_settings,
        name='list_settings',
    ),
    re_path(
        r'^unsubscribe_all$',
        list_views.remove_all_subscribers,
        name='unsubscribe_all',
    ),
    re_path(r'^confirm/$', list_views.confirm_token, name='confirm_token'),
    re_path(
        r'^templates$',
        template_views.ListTemplateIndexView.as_view(),
        name='list_template_list',
    ),
    re_path(
        r'^templates/new$',
        template_views.ListTemplateCreateView.as_view(),
        name='list_template_new',
    ),
    re_path(
        r'^templates/(?P<pk>[^/]+)?/update$',
        template_views.ListTemplateUpdateView.as_view(),
        name='list_template_update',
    ),
    re_path(
        r'^templates/(?P<pk>[^/]+)?/delete$',
        template_views.ListTemplateDeleteView.as_view(),
        name='list_template_delete',
    ),
]

urlpatterns = [
    re_path(r'^$', list_views.list_index),  # noqa: W605 (bogus)
    re_path(
        r'^accounts/subscriptions/$',
        user_views.user_subscriptions,
        name='ps_user_profile',
    ),
    re_path(
        r'^accounts/per-address-preferences/$',
        user_views.UserAddressPreferencesView.as_view(),
        name='user_address_preferences',
    ),
    # if this URL changes, update Mailman's Member.options_url
    re_path(
        r'^accounts/per-subscription-preferences/$',
        user_views.UserSubscriptionPreferencesView.as_view(),
        name='user_subscription_preferences',
    ),
    re_path(
        r'^accounts/mailmansettings/$',
        user_views.UserMailmanSettingsView.as_view(),
        name='user_mailmansettings',
    ),
    re_path(
        r'^accounts/list-options/(?P<member_id>[^/]+)/$',
        user_views.UserListOptionsView.as_view(),
        name='user_list_options',
    ),
    # /domains/
    re_path(r'^domains/$', domain_views.domain_index, name='domain_index'),
    re_path(r'^domains/new/$', domain_views.domain_new, name='domain_new'),
    re_path(
        r'^domains/(?P<domain>[^/]+)/$',
        domain_views.domain_edit,
        name='domain_edit',
    ),
    re_path(
        r'^domains/(?P<domain>[^/]+)/delete$',
        domain_views.domain_delete,
        name='domain_delete',
    ),
    re_path(
        r'^domains/(?P<domain>[^/]+)/owners$',
        domain_views.domain_owners,
        name='domain_owners',
    ),
    re_path(
        r'^domains/(?P<domain>[^/]+)/owners/(?P<user_id>.+)/remove$',
        domain_views.remove_owners,
        name='remove_domain_owner',
    ),
    # Ideally, these paths should be accessible by domain_owners, however,
    # we don't have good ways to check that, so for now, this views are
    # protected by superuser privileges.
    # I know it is bad, but this will be fixed soon. See postorius#
    re_path(
        r'^domains/(?P<domain>[^/]+)/templates$',
        template_views.DomainTemplateIndexView.as_view(),
        name='domain_template_list',
    ),
    re_path(
        r'^domains/(?P<domain>[^/]+)/templates/new$',
        template_views.DomainTemplateCreateView.as_view(),
        name='domain_template_new',
    ),
    re_path(
        r'^domains/(?P<domain>[^/]+)/templates/(?P<pk>[^/]+)/update$',  # noqa: E501
        template_views.DomainTemplateUpdateView.as_view(),
        name='domain_template_update',
    ),
    re_path(
        r'^domains/(?P<domain>[^/]+)/templates/(?P<pk>[^/]+)/delete$',  # noqa: E501
        template_views.DomainTemplateDeleteView.as_view(),
        name='domain_template_delete',
    ),
    # /lists/
    re_path(r'^lists/$', list_views.list_index, name='list_index'),
    re_path(r'^lists/new/$', list_views.list_new, name='list_new'),
    re_path(r'^lists/(?P<list_id>[^/]+)/', include(list_patterns)),
    # /system/
    re_path(
        r'^system/$',
        system_views.system_information,
        name='system_information',
    ),
    # /bans/
    re_path(r'^bans/$', system_views.bans, name='global_bans'),
    # /api/
    re_path(
        r'^api/list/(?P<list_id>[^/]+)/held_message/(?P<held_id>\d+)/$',  # noqa: E501
        rest_views.get_held_message,
        name='rest_held_message',
    ),
    re_path(
        r'^api/list/(?P<list_id>[^/]+)/held_message/(?P<held_id>\d+)/'
        r'attachment/(?P<attachment_id>\d+)/$',
        rest_views.get_attachment_for_held_message,
        name='rest_attachment_for_held_message',
    ),
    # URL configuration for templates.
    re_path(
        r'^api/templates/(?P<context>[^/]+)/(?P<identifier>[^/]+)/(?P<name>[^/]+)',  # noqa: E501
        template_views.get_template_data,
        name='rest_template',
    ),
    # users.
    re_path(r'users$', user_views.list_users, name='list_users'),
    re_path(
        r'users/(?P<user_id>[^/]+)/manage$',
        user_views.manage_user,
        name='manage_user',
    ),
    re_path(
        r'users/(?P<user_id>[^/]+)/delete$',
        user_views.delete_user,
        name='delete_user',
    ),
]



# Generated by Django 4.2.5 on 2023-11-04 02:49

from django.db import migrations, models


# flake8: noqa

class Migration(migrations.Migration):

    dependencies = [
        ('postorius', '0018_alter_emailtemplate_language'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailtemplate',
            name='data',
            field=models.TextField(blank=True, help_text='Note: Do not add any secret content in templates as they are publicly accessible.\nYou can use these variables in the templates. \n$hyperkitty_url: Permalink to archived message in Hyperkitty  - only in list:member:regular:footer and list:member:regular:header\n$listname: Name of the Mailing List e.g. ant@example.com \n$list_id: The List-ID header e.g. ant.example.com \n$display_name: Display name of the mailing list e.g. Ant \n$short_listname: Local part of the listname e.g. ant \n$domain: The domain part of the listname e.g. example.com \n$description: The mailing list\'s short description text \n$info: The mailing list\'s longer descriptive text \n$request_email: The email address for -request address \n$owner_email: The email address for -owner address \n$site_email: The email address to reach the owners of the site \n$language: The two letter language code for list\'s preferred language e.g. fr, en, de \nSome templates accept additional variables. See <a href="https://docs.mailman3.org/projects/mailman/en/latest/src/mailman/rest/docs/templates.html#templated-texts">this doc</a> \n'),
        ),
    ]



# -*- coding: utf-8 -*-
# Copyright (C) 2018-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#


from django.contrib.auth.mixins import LoginRequiredMixin, UserPassesTestMixin

from postorius.auth.utils import set_domain_access_props, set_list_access_props
from postorius.models import List


class ListOwnerMixin(LoginRequiredMixin, UserPassesTestMixin):
    """Mixin to allow access to only List owners."""

    raise_exception = True

    def test_func(self):
        user = self.request.user
        mlist_id = self.kwargs['list_id']
        mlist = List.objects.get_or_404(mlist_id)
        if user.is_superuser:
            return True
        set_list_access_props(user, mlist, moderator=False)
        return user.is_list_owner


class ListModeratorMixin(LoginRequiredMixin, UserPassesTestMixin):
    """Mixin to allow access to only List Moderators."""

    raise_exception = True

    def test_func(self):
        user = self.request.user
        mlist_id = self.kwargs['list_id']
        if user.is_superuser:
            return True
        set_list_access_props(user, mlist_id)
        return user.is_list_owner or user.is_list_moderator


class UserShowListMembersMixin(LoginRequiredMixin, UserPassesTestMixin):
    """Mixin to allow listing of members depending on list settings and
    user role."""

    raise_exception = True

    def test_func(self):
        user = self.request.user
        if user.is_superuser:
            return True
        mlist_id = self.kwargs['list_id']
        set_list_access_props(user, mlist_id)
        return user.show_list_members


class DomainOwnerMixin(LoginRequiredMixin, UserPassesTestMixin):
    """Mixin to allow access to only Domain Owner."""

    raise_exception = True

    def test_func(self):
        user = self.request.user
        domain = self.kwargs['domain']
        if user.is_superuser:
            return True
        set_domain_access_props(user, domain)
        return user.is_domain_owner


class SuperUserRequiredMixin(LoginRequiredMixin, UserPassesTestMixin):
    """Mixin to allow access to only Django Superusers."""

    def test_func(self):
        return self.request.user.is_superuser



# Generated by Django 2.1.7 on 2019-03-10 07:17

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('postorius', '0007_auto_20180712_0536'),
    ]

    operations = [
        migrations.RenameField(
            model_name='emailtemplate',
            old_name='craeted_at',
            new_name='created_at',
        ),
    ]



# -*- coding: utf-8 -*-

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('postorius', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='addressconfirmationprofile',
            name='activation_key',
            field=models.CharField(unique=True, max_length=32),
        ),
        migrations.AlterField(
            model_name='addressconfirmationprofile',
            name='created',
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AlterField(
            model_name='addressconfirmationprofile',
            name='email',
            field=models.EmailField(unique=True, max_length=254),
        ),
    ]



# -*- coding: utf-8 -*-
# Copyright (C) 1998-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.

"""
Authentication and authorization-related utilities.
"""

from allauth.account.models import EmailAddress

from postorius.models import Domain, List


ALL_ROSTER = ['owner', 'moderator', 'member', 'nomember']


def user_is_in_list_roster(user, mailing_list, roster):
    """Checks if a user is in a MailingList roster.

    :param user: User to check access permissions for.
    :type user: django.contrib.auth.model.User
    :param mailing_list: MailingList to check permissions for.
    :type mailing_list: postorius.models.List
    :param roster: Access permissions required.
    :type roster: str
    """
    if not user.is_authenticated:
        return False
    addresses = set(
        email.lower()
        for email in EmailAddress.objects.filter(
            user=user, verified=True
        ).values_list('email', flat=True)
    )
    if roster not in ALL_ROSTER:
        raise ValueError(f'{roster} is a valid List Roster.')

    roster_addresses = set(
        [
            member.email.lower()
            for member in mailing_list.get_roster(roster, fields=['email'])
        ]
    )
    if addresses & roster_addresses:
        return True  # At least one address is in the roster
    return False


def set_list_access_props(user, mlist, owner=True, moderator=True):
    """Update user's access permissions of a MailingList.

    :param user: The user to check permissions for.
    :type user: django.contrib.auth.model.User
    :param mlist: MailingList to check permissions for.
    :type mlist: postorius.models.List
    :param owner: Set is_list_owner.
    :type owner: bool
    :param moderator: Set is_list_moderator.
    :type moderator: bool
    """
    # If given a mailinglist id, get the List object instead.
    if isinstance(mlist, str):
        mlist = List.objects.get_or_404(mlist)
    # If not already set, check if the user is in list ownership roster.
    if (not hasattr(user, 'is_list_owner')) and owner:
        user.is_list_owner = user_is_in_list_roster(user, mlist, 'owner')
    # Calculate combined status for superusers and list owners
    user.is_poweruser = user.is_superuser or user.is_list_owner
    # If not already set, check if the user is in list moderator roster.
    if not hasattr(user, 'is_list_moderator') and moderator:
        user.is_list_moderator = user_is_in_list_roster(
            user, mlist, 'moderator'
        )
    if not hasattr(user, 'show_list_members'):
        member_roster_visibility = mlist.settings['member_roster_visibility']
        if user.is_superuser or user.is_list_owner:
            user.show_list_members = True
        else:
            is_list_moderator = (
                user.is_list_moderator
                if hasattr(user, 'is_list_moderator')
                else user_is_in_list_roster(user, mlist, 'moderator')
            )
            user.show_list_members = (
                member_roster_visibility == 'moderators' and is_list_moderator
            ) or (
                member_roster_visibility == 'members'
                and (
                    is_list_moderator
                    or user_is_in_list_roster(user, mlist, 'member')
                )
            )


def set_domain_access_props(user, domain):
    """Update user's access permissions for a domain.

    :param user: The user to check permissions for.
    :type user: django.contrib.auth.model.User
    :param domain: Domain to check permissions for.
    :type domain: postorius.models.Domain
    """
    # TODO: This is very slow as it involves first iterating over every domain
    # owner and then each of their addresses. Create an API in Core to
    # facilitate this.
    if isinstance(domain, str):
        domain = Domain.objects.get_or_404(domain)
    owner_addresses = []
    for owner in domain.owners:
        owner_addresses.extend(owner.addresses)
    owner_addresses = set([each.email for each in owner_addresses])
    user_addresses = set(
        EmailAddress.objects.filter(user=user, verified=True).values_list(
            'email', flat=True
        )
    )
    user.is_domain_owner = owner_addresses & user_addresses



# -*- coding: utf-8 -*-
# Copyright (C) 2020-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#

import logging

from django.dispatch import receiver

from django_mailman3.models import MailDomain
from django_mailman3.signals import domain_created

from postorius.models import Domain


log = logging.getLogger(__name__)


@receiver(domain_created, sender=Domain)
def maybe_update_site(sender, **kw):
    """Update the Site's name and description match the domain iff it is
    example.com.

    On a new Django instance, the default Site is example.com and when a user
    creates a new domain, they'll still see example.com in the Title.  It is
    safe to update the Site object to the domain's mail host and description.

    Do not touch the Site object if it isn't example.com.

    :param sender: The model class.
    :param instance: The actual instance of the sender class.
    :param created: IF the new record was created.
    """
    mail_host = kw.get('mail_host')

    mail_domain = MailDomain.objects.get(mail_domain=mail_host)
    domain = Domain.objects.get(mail_host=mail_host)
    site = mail_domain.site
    if not site.domain == 'example.com':
        log.info('Not updating site domain')
        return
    log.info('Updating site domain.')
    # Update the Site to match the domain with mail_host and description to
    # description if not empty else mail_host.
    site.domain = domain.mail_host
    site.name = domain.description or domain.mail_host
    site.save()
    log.debug('Update Site %s to match domain %s', site, domain)
